#include <time.h>
#include "e_mod_main.h"
#include <string.h>
#include <stdio.h>
#include <libintl.h>

/* module private routines */
static Calendar  	*_calendar_new();
static void   		_calendar_shutdown(Calendar *calendar);
static void    		_calendar_config_menu_new(Calendar *calendar);
static void 		_calendar_add_dates(Calendar_Face *face, Calendar *calendar, int skew);

static Calendar_Face 	*_calendar_face_new(E_Container *con,Calendar *calendar);
static void   	 	_calendar_face_free(Calendar_Face *face);
static void   		 _calendar_face_menu_new(Calendar_Face *face, Calendar *calendar);
static int     		_calendar_face_config_cb_timer(void *data);
static void    		_calendar_face_cb_menu_edit(void *data, E_Menu *m,E_Menu_Item *mi);
static void 		_calendar_face_start(Calendar *calendar);
static void 		_calendar_face_cb_mouse_down(void *data, Evas *e, Evas_Object *obj,
                                        void *event_info);
static void   		 _calendar_face_cb_gmc_change(void *data, E_Gadman_Client *gmc, 
					   E_Gadman_Change change);

static int 		_date_cb_check(void *calendar);
static void 		_clear_dates(Calendar_Face *face);
static void 		_read_conf_files(void *data, E_Menu *m, E_Menu_Item *mi);
static int		_calculate_skew(int month, Calendar *calendar);
static void		_fill_in_caltable(int FirstDay,Calendar_Face *faces, int StartUp);

static int _calendar_count;
static E_Config_DD *conf_edd;
static E_Config_DD *conf_font_edd;
static E_Config_DD *conf_color_edd;

//void (*pt2Function)(void *data, void *data2) = NULL;
E_Module_Api e_modapi =
{
   E_MODULE_API_VERSION,
   "Calendar"
};
/************ public module routines. all modules must have these ***************/
/***************************************************
/ Function: 
/ Purpose:  
/ Takes: 
/
/
/*****************************************************/
void *
e_modapi_init(E_Module *module)
{
   Calendar *calendar;
   /* check module api version */
   if (module->api->version < E_MODULE_API_VERSION)
     {
	e_error_dialog_show
	  ("Module API Error",
	   "Error initializing Module: calendar\n"
	   "It requires a minimum module API version of: %i.\n"
	   "The module API advertized by Enlightenment is: %i.\n"
	   "Aborting module.",
	   E_MODULE_API_VERSION, module->api->version);
	return NULL;
     }
   /* actually init buttons */
   calendar = _calendar_new();
   module->config_menu = calendar->config_menu;
   return calendar;
}
/***************************************************
/ Function: 
/ Purpose:  
/
/*****************************************************/
/*    * int e_modapi_shutdown(E_Module*) - Cleanup */
int
e_modapi_shutdown(E_Module *module)
{
   Calendar *calendar;

   if (module->config_menu)
     module->config_menu = NULL;

   calendar = module->data;
   if (calendar)
     _calendar_shutdown(calendar);

   return 1;
}
/***************************************************
/ Function: 
/ Purpose:  
/
/*****************************************************/
/*    * int e_modapi_save(E_Module*) - Save persistent data */
/*    ie: the calendar->conf stuff                          */
int
e_modapi_save(E_Module *module)
{
   Calendar *calendar;

   calendar = module->data;
   e_config_domain_save("module.calendar", conf_edd, calendar->conf);
   return 1;
}
/***************************************************
/ Function: 
/ Purpose:  
/
/*****************************************************/
/* * int e_modapi_info(E_Module*) - Setup module specific infomation */
int
e_modapi_info(E_Module *module)
{
   module->icon_file = strdup(PACKAGE_LIB_DIR "/calendar/module_icon.png");
   return 1;
}
/***************************************************
/ Function: 
/ Purpose:  
/
/
/*****************************************************/
/* * int e_modapi_about(E_Module*). - Called when Modules' About Menu is invoked. */
int
e_modapi_about(E_Module *module)
{
   e_error_dialog_show(_("Enlightenment calendar Module"),
		       _("A simple module to give E17 a desktop calendar "));
   return 1;
}
/************************End of the required routines **********************************************/
/************************End of the required routines **********************************************/


/* module private routines */
/***************************************************
/ Function: _calendar_new()
/ Purpose:  create new calendar
/
/
/*****************************************************/
static Calendar *
_calendar_new()
{
   Calendar *calendar;//creates new _calendar struc. [faces(Evas_list), config_menu(E_menu) & config(Config)]
   time_t now;
   struct tm date;
   time(&now);
   date = *localtime(&now);
 
   _calendar_count = 0;
   calendar = E_NEW(Calendar, 1);
   if (!calendar) return NULL;

   conf_font_edd = E_CONFIG_DD_NEW("CalFonts",CalFonts);
#undef T
#undef D
#define T CalFonts
#define D conf_font_edd
   E_CONFIG_VAL(D, T, font, STR);
   E_CONFIG_VAL(D, T, size, INT);

   conf_color_edd = E_CONFIG_DD_NEW("c_array",c_array);
#undef T
#undef D
#define T c_array
#define D conf_color_edd
   E_CONFIG_VAL(D, T, red, INT);
   E_CONFIG_VAL(D, T, green, INT);
   E_CONFIG_VAL(D, T, blue, INT);
   E_CONFIG_VAL(D, T, alpha, INT);
   E_CONFIG_VAL(D, T, red_o, INT);
   E_CONFIG_VAL(D, T, green_o, INT);
   E_CONFIG_VAL(D, T, blue_o, INT);
   E_CONFIG_VAL(D, T, alpha_o, INT);
   E_CONFIG_VAL(D, T, red_s, INT);
   E_CONFIG_VAL(D, T, green_s, INT);
   E_CONFIG_VAL(D, T, blue_s, INT);
   E_CONFIG_VAL(D, T, alpha_s, INT); 

   conf_edd = E_CONFIG_DD_NEW("calendar_Config", Config);
#undef T
#undef D
#define T Config
#define D conf_edd
   E_CONFIG_VAL(D, T, ImageYes, INT);
   E_CONFIG_VAL(D, T, config_version, INT);
   E_CONFIG_VAL(D, T, DayofWeek_Start, INT);
   E_CONFIG_VAL(D, T, TopImage_path, STR);
   E_CONFIG_VAL(D, T, Background_path, STR);
   E_CONFIG_VAL(D, T, ConfigFile_path, STR);
   E_CONFIG_VAL(D, T, label_path, STR);
   E_CONFIG_VAL(D, T, today_path, STR);
   E_CONFIG_VAL(D, T, weekend_path, STR);
   E_CONFIG_VAL(D, T, weekday_path, STR);
   E_CONFIG_VAL(D, T, UserCS, INT);
//Store colors
   E_CONFIG_LIST(D, T, Today_s_text_colors, conf_color_edd);
   E_CONFIG_LIST(D, T, Today_s_back_colors, conf_color_edd);
   E_CONFIG_LIST(D, T, Weekend_text_colors, conf_color_edd);
   E_CONFIG_LIST(D, T, WeekDay_text_colors, conf_color_edd);
   E_CONFIG_LIST(D, T, Weekend_back_colors, conf_color_edd);
   E_CONFIG_LIST(D, T, WeekDay_back_colors, conf_color_edd);
   E_CONFIG_LIST(D, T, YearMon_numb_colors, conf_color_edd);
   E_CONFIG_LIST(D, T, DayWeek_text_colors, conf_color_edd);
//Store fonts
   E_CONFIG_LIST(D, T, YM_text_class, conf_font_edd);
   E_CONFIG_LIST(D, T, Day_text_class, conf_font_edd);
   E_CONFIG_LIST(D, T, text_class, conf_font_edd);

   calendar->conf = e_config_domain_load("module.calendar", conf_edd);
   if (!calendar->conf)
   {
	c_array *Temp_Ptr, *Temp_Ptr1,*Temp_Ptr2,*Temp_Ptr3,*Temp_Ptr4,*Temp_Ptr5,*Temp_Ptr6,*Temp_Ptr7;
     	Temp_Ptr = E_NEW(c_array,1);
        Temp_Ptr->red = -1;
     	Temp_Ptr1 = E_NEW(c_array,1);
        Temp_Ptr1->red = -1;
     	Temp_Ptr2 = E_NEW(c_array,1);
        Temp_Ptr2->red = -1;
     	Temp_Ptr3 = E_NEW(c_array,1);
        Temp_Ptr3->red = -1;
     	Temp_Ptr4 = E_NEW(c_array,1);
        Temp_Ptr4->red = -1;
     	Temp_Ptr5 = E_NEW(c_array,1);
        Temp_Ptr5->red = -1;
     	Temp_Ptr6 = E_NEW(c_array,1);
        Temp_Ptr6->red = -1;
     	Temp_Ptr7 = E_NEW(c_array,1);
        Temp_Ptr7->red = -1;

	CalFonts *Temp_ClassPtr;
	Temp_ClassPtr = E_NEW(CalFonts,1);
	Temp_ClassPtr->font = "";

	CalFonts *Temp_ClassPtr1;
	Temp_ClassPtr1 = E_NEW(CalFonts,1);
	Temp_ClassPtr1->font = "";

	CalFonts *Temp_ClassPtr2;
	Temp_ClassPtr2 = E_NEW(CalFonts,1);
	Temp_ClassPtr2->font = "";


	calendar->conf = E_NEW(Config, 1);
        calendar->conf->ImageYes = 0;
	calendar->conf->TopImage_path = strdup(PACKAGE_DATA_DIR"/topimage.edj");
	calendar->conf->Background_path = strdup(PACKAGE_DATA_DIR"/backimage.edj");
	calendar->conf->today_path = strdup(PACKAGE_DATA_DIR"/today.edj");
	calendar->conf->weekend_path = strdup(PACKAGE_DATA_DIR"/weekend.edj");
	calendar->conf->weekday_path = strdup(PACKAGE_DATA_DIR"/weekday.edj");
	calendar->conf->label_path = strdup(PACKAGE_DATA_DIR"/label.edj");
	calendar->conf->ConfigFile_path = "";	

        calendar->conf->Today_s_text_colors = evas_list_append(calendar->conf->Today_s_text_colors, Temp_Ptr);
        calendar->conf->Today_s_back_colors = evas_list_append(calendar->conf->Today_s_back_colors, Temp_Ptr1);
        calendar->conf->Weekend_text_colors = evas_list_append(calendar->conf->Weekend_text_colors, Temp_Ptr2);
        calendar->conf->Weekend_back_colors = evas_list_append(calendar->conf->Weekend_back_colors, Temp_Ptr3);
        calendar->conf->WeekDay_text_colors = evas_list_append(calendar->conf->WeekDay_text_colors, Temp_Ptr4);
        calendar->conf->WeekDay_back_colors = evas_list_append(calendar->conf->WeekDay_back_colors, Temp_Ptr5);
        calendar->conf->YearMon_numb_colors = evas_list_append(calendar->conf->YearMon_numb_colors, Temp_Ptr6);
        calendar->conf->DayWeek_text_colors = evas_list_append(calendar->conf->DayWeek_text_colors, Temp_Ptr7);

	calendar->conf->YM_text_class = evas_list_append(calendar->conf->YM_text_class, Temp_ClassPtr);
	calendar->conf->Day_text_class = evas_list_append(calendar->conf->Day_text_class, Temp_ClassPtr1);
	calendar->conf->text_class = evas_list_append(calendar->conf->text_class, Temp_ClassPtr2);

	calendar->conf->DayofWeek_Start = 0;
	calendar->conf->UserCS = FALSE;

   	calendar_face_set_colors(calendar);
   	calendar_face_set_text(calendar);
   }
   else
   {
	if (strncmp(calendar->conf->TopImage_path," ",2) == 0)
  	{
		calendar->conf->TopImage_path = strdup(PACKAGE_DATA_DIR"/topimage.edj");
		calendar->conf->Background_path = strdup(PACKAGE_DATA_DIR"/backimage.edj");
		calendar->conf->today_path = strdup(PACKAGE_DATA_DIR"/today.edj");
		calendar->conf->weekend_path = strdup(PACKAGE_DATA_DIR"/weekend.edj");
		calendar->conf->weekday_path = strdup(PACKAGE_DATA_DIR"/weekday.edj");
		calendar->conf->label_path = strdup(PACKAGE_DATA_DIR"/label.edj");
   	}

   }
   calendar->conf->Today = date.tm_mday;
//Start up Face
   _calendar_face_start(calendar);
//Add Timer
   calendar->date_check_timer = ecore_timer_add(60, _date_cb_check,calendar);
   return calendar;
}
/***************************************************
/ Function: 
/ Purpose:  
/
/
/*****************************************************/ 
static void
_calendar_face_start(Calendar *calendar)
{
   Evas_List *cl, *managers, *l2, *l;  
   E_Menu_Item *mi;
   _calendar_config_menu_new(calendar);
   managers = e_manager_list();
   for (l = managers; l; l = l->next)
     {
	E_Manager *man;
	
	man = l->data;
	for (l2 = man->containers; l2; l2 = l2->next)
	  {
	     E_Container *con;
	     Calendar_Face *face;

	     con = l2->data;
	     face = _calendar_face_new(con,calendar);
	     if (face)
	       {
		  calendar->faces = evas_list_append(calendar->faces, face);
    		  _calendar_face_menu_new(face,calendar);
   		  mi = e_menu_item_new(calendar->config_menu);
    		  e_menu_item_label_set(mi, con->name);
    		  e_menu_item_submenu_set(mi, face->menu);

	       }
 	     /*	_clear_dates(face);
		update_colors(calendar,face);
		calendar_update_text_fonts(calendar, face); */
 	  }
     }

 }
/***************************************************
/ Function: _calendar_shutdown(Calendar *calendar)
/ Purpose:  shutdown calendar module
/
/
/*****************************************************/ 
static void
_calendar_shutdown(Calendar *calendar)
{
   Evas_List *list;

   E_CONFIG_DD_FREE(conf_edd);
   E_CONFIG_DD_FREE(conf_font_edd);
   E_CONFIG_DD_FREE(conf_color_edd);
   ecore_timer_del(calendar->date_check_timer);

   for (list = calendar->faces; list; list = list->next)
     _calendar_face_free(list->data);
   evas_list_free(calendar->faces);
   e_object_del(E_OBJECT(calendar->config_menu));

  // need to free color & fonts list
   evas_list_free(calendar->conf->Today_s_text_colors);
   evas_list_free(calendar->conf->Today_s_back_colors);
   evas_list_free(calendar->conf->Weekend_text_colors);
   evas_list_free(calendar->conf->Weekend_back_colors);
   evas_list_free(calendar->conf->WeekDay_text_colors);
   evas_list_free(calendar->conf->WeekDay_back_colors);
   evas_list_free(calendar->conf->YearMon_numb_colors);
   evas_list_free(calendar->conf->DayWeek_text_colors);
   evas_list_free(calendar->conf->YM_text_class);
   evas_list_free(calendar->conf->Day_text_class);
   evas_list_free(calendar->conf->text_class);

   free(calendar->conf);
   free(calendar);
}
/***************************************************
/ Function: _calendar_config_menu_new(Calendar *calendar)
/ Purpose:  create new calender menu
/
/
/*****************************************************/
static void
_calendar_config_menu_new(Calendar *calendar)
{
   calendar->config_menu = e_menu_new();
}
/***************************************************
/ Function: 
/ Purpose: 
/
/*****************************************************/
static Calendar_Face *
_calendar_face_new(E_Container *con,Calendar *calendar)
{
   // Setup date finding stuff
   time_t now;
   struct tm date;
   time(&now);
   date = *localtime(&now);
   int check,i;

   Calendar_Face *face;
   Evas_Object *o3, *o_table, *o_table2; 
   Evas_Object *today;

   face = E_NEW(Calendar_Face, 1);
   if (!face) return NULL;
   face->con = con; 
   e_object_ref(E_OBJECT(con));
   evas_event_freeze(con->bg_evas);

   // Make calendar object
   face->calendar_object = edje_object_add(con->bg_evas);
   check = edje_object_file_set(face->calendar_object, PACKAGE_DATA_DIR"/calendar.edj", "default");
   evas_object_show(face->calendar_object);

   //create first table that will be swallowed into edje
   face->table_object = e_table_add(con->bg_evas);
   e_table_homogenous_set(face->table_object, 1); // 1 = homo, 0 = nonhomo
   edje_object_part_swallow(face->calendar_object, "swallow", face->table_object);
   evas_object_show(face->table_object); 

   //Make topimage object
   face->topimage_object = edje_object_add(con->bg_evas);
   check = edje_object_file_set(face->topimage_object, calendar->conf->TopImage_path,
									"default");
    //Make background object
   face->background_object = edje_object_add(con->bg_evas); 
   check = edje_object_file_set(face->background_object, calendar->conf->Background_path,
									"back");

   //Make label object
   face->label_object = edje_object_add(con->bg_evas);
   if (!calendar->conf->UserCS)
   	check = edje_object_file_set(face->label_object, calendar->conf->label_path,
								"default");
   else
   	check = edje_object_file_set(face->label_object, calendar->conf->label_path,
								"user");
   set_day_label(face->label_object, calendar->conf->DayofWeek_Start, date.tm_mon, date.tm_year);

  if (calendar->conf->ImageYes == 0)
   { 	
        //Setup Top Image -- Puts topimage in top 3 rows

   	e_table_pack(face->table_object, face->topimage_object, 0, 0, 1, 3); 
  	e_table_pack_options_set(face->topimage_object, 1, 1, 1, 1, 0.5, 0.5, 0, 0, -1, -1);
	evas_object_show(face->topimage_object);
	// Puts label row in 4 row
   	e_table_pack(face->table_object, face->label_object, 0, 4, 1, 1);
  	e_table_pack_options_set(face->label_object,1, 1, 1, 1, 0.5, 0.5, 0, 0, -1, -1);
	evas_object_show(face->label_object);
        // Puts background in rows 5 - 9
   	e_table_pack(face->table_object, face->background_object, 0, 5, 1, 5);
  	e_table_pack_options_set(face->background_object,1, 1, 1, 1, 0.5, 0.5, 0, 0, -1, -1);
	evas_object_show(face->background_object);
   }
   else
   {
	// Puts label row in 1 row
   	e_table_pack(face->table_object, face->label_object, 0, 0, 1, 1);
  	e_table_pack_options_set(face->label_object,1, 1, 1, 1, 0.5, 0.5, 0, 0, -1, -1);
	evas_object_show(face->label_object);
        //Setup background -- put it in the back of the table (2nd row as no topimage)
   	e_table_pack(face->table_object, face->background_object, 0, 1, 1, 5);
  	e_table_pack_options_set(face->background_object,1, 1, 1, 1, 0.5, 0.5, 0, 0, -1, -1);
	evas_object_show(face->background_object);
    }

   //Make day objects
   i = 0;
   for (i;i<35;i++)
   {
   	face->weekend_object[i] = edje_object_add(con->bg_evas);
	if (!calendar->conf->UserCS)
    	   check = edje_object_file_set(face->weekend_object[i], calendar->conf->weekend_path,
										   "default");
	else
    	   check = edje_object_file_set(face->weekend_object[i], calendar->conf->weekend_path,
											"user");
    	evas_object_event_callback_add(face->weekend_object[i], EVAS_CALLBACK_MOUSE_DOWN,
                                                      _calendar_face_cb_mouse_down, face);

	face->weekday_object[i] = edje_object_add(con->bg_evas);
	if (!calendar->conf->UserCS)
   	   check = edje_object_file_set(face->weekday_object[i], calendar->conf->weekday_path,
										   "default");
	else
   	   check = edje_object_file_set(face->weekday_object[i], calendar->conf->weekday_path,
										      "user");
    	evas_object_event_callback_add(face->weekday_object[i], EVAS_CALLBACK_MOUSE_DOWN,
                                                      _calendar_face_cb_mouse_down, face);

	face->today_object[i] = edje_object_add(con->bg_evas);
   	if (!calendar->conf->UserCS)
   		check = edje_object_file_set(face->today_object[i], calendar->conf->today_path,"default");
   	else
   		check = edje_object_file_set(face->today_object[i], calendar->conf->today_path, "user");
  	evas_object_event_callback_add(today, EVAS_CALLBACK_MOUSE_DOWN, _calendar_face_cb_mouse_down, face);
   }

   /* setup calendar object */
   face->o_calendar_table = e_table_add(con->bg_evas);
   e_table_homogenous_set(face->o_calendar_table, 1);
   edje_object_part_swallow(face->background_object, "swallow", face->o_calendar_table);
   evas_object_show(face->o_calendar_table); 
    //Time to do some packing....
   int x = 0;
   i = 0;
   int row = 0;
   while (x<35)
   {
     for (i; i<7;i++)
     {
	e_table_pack(face->o_calendar_table,face->weekend_object[x], i, row, 1, 1);
	e_table_pack_options_set(face->weekend_object[x],1, 1, 1, 1, 0.5, 0.5, 0, 0, -1, -1);

	e_table_pack(face->o_calendar_table,face->weekday_object[x], i, row, 1, 1);
	e_table_pack_options_set(face->weekday_object[x],1, 1, 1, 1, 0.5, 0.5, 0, 0, -1, -1);

	e_table_pack(face->o_calendar_table,face->today_object[x], i, row, 1, 1);
	e_table_pack_options_set(face->today_object[x],1, 1, 1, 1, 0.5, 0.5, 0, 0, -1, -1);
	x++; 
     }
     i = 0;
     row++;
   } 

   _fill_in_caltable(calendar->conf->DayofWeek_Start, face, 0);

   /* setup gadman */
   face->gmc = e_gadman_client_new(con->gadman);
   e_gadman_client_domain_set(face->gmc, "module.calendar", _calendar_count++);
   e_gadman_client_policy_set(face->gmc,
			      E_GADMAN_POLICY_ANYWHERE |
			      E_GADMAN_POLICY_HMOVE |
			      E_GADMAN_POLICY_VMOVE |
			      E_GADMAN_POLICY_HSIZE |
			      E_GADMAN_POLICY_VSIZE);
   //e_gadman_client_min_size_set(face->gmc, 14, 7);
   e_gadman_client_align_set(face->gmc, 1.0, 1.0);
   e_gadman_client_resize(face->gmc, 240, 320);
   e_gadman_client_change_func_set(face->gmc, _calendar_face_cb_gmc_change,face);
   e_gadman_client_load(face->gmc);

   evas_event_thaw(con->bg_evas);

   int skew = _calculate_skew(date.tm_mon,calendar);
   _calendar_add_dates(face,calendar,skew);
   calendar_face_set_text(calendar);
   if (calendar->conf->UserCS)
   {
   	calendar_update_text_fonts(calendar,face);
   	update_colors(calendar,face);
   }
   return face;
}
/***************************************************
/ Function: _calendar_face_free(Calendar_Face *face)
/ Purpose:  
/
/
/*****************************************************/
static void
_calendar_face_free(Calendar_Face *face)
{
   int i = 0;
    for (i;i<35;i++)
   {
   	evas_object_del(face->weekend_object[i]);
	evas_object_del(face->weekday_object[i]); 
   	evas_object_del(face->today_object[i]);
   }
   if (face->con) e_object_unref(E_OBJECT(face->con));
   e_object_del(E_OBJECT(face->gmc));
   evas_object_del(face->topimage_object);
   evas_object_del(face->calendar_object);
   evas_object_del(face->label_object);
   evas_object_del(face->background_object);
   evas_object_del(face->o_calendar_table);
   evas_object_del(face->table_object);
   e_object_del(E_OBJECT(face->menu));
   if (face->sub) e_object_del(E_OBJECT(face->sub));
   if (face->sub) e_object_del(E_OBJECT(face->sub2));
   e_object_del(E_OBJECT(face->sub3));
   free(face);
   _calendar_count--;
}
/***************************************************
/ Function: _calendar_face_menu_new(Calendar_Face *face,Calendar *calendar)
/ Purpose:  
/ Returns: nothing
/
/*****************************************************/
static void
_calendar_face_menu_new(Calendar_Face *face, Calendar *calendar)
{
   E_Menu_Item *mi;
   E_Menu_Item *sub, *sub2, *sub3;
   Ecore_List* interfaces = NULL;

   c_array *TBC_Ptr = calendar->conf->Today_s_back_colors->data;
   c_array *TTC_Ptr = calendar->conf->Today_s_text_colors->data;
   c_array *WeTC_Ptr = calendar->conf->Weekend_text_colors->data; 
   c_array *WdTC_Ptr = calendar->conf->WeekDay_text_colors->data; 
   c_array *WeBC_Ptr = calendar->conf->Weekend_back_colors->data; 
   c_array *WdBC_Ptr = calendar->conf->WeekDay_back_colors->data; 
   c_array *YNC_Ptr = calendar->conf->YearMon_numb_colors->data;  
   c_array *DTC_Ptr = calendar->conf->DayWeek_text_colors->data;  

   CalFonts *YTC_Ptr = calendar->conf->YM_text_class->data;     
   CalFonts *DTCs_Ptr = calendar->conf->Day_text_class->data;      
   CalFonts *TC_Ptr  = calendar->conf->text_class->data;           


   int interface_count = 0;
   char* interface_name = NULL;
   /* Setup Menus */
   face->menu = e_menu_new();
   /* Main Menu Items */
//Add Edit Mode option
   mi = e_menu_item_new(face->menu);
   e_menu_item_label_set(mi, _("Edit Mode"));
   e_menu_item_callback_set(mi, _calendar_face_cb_menu_edit, face);
   if (calendar->conf->UserCS)
   {
//Add option to read config file
   	mi = e_menu_item_new(face->menu);
   	e_menu_item_label_set(mi, _("Read Conf Files"));
   	e_menu_item_callback_set(mi, _read_conf_files, calendar); 
   }
//Add option to read config file
   mi = e_menu_item_new(face->menu);
   e_menu_item_label_set(mi, _("Config Dialog"));
   e_menu_item_callback_set(mi, config_dialog, calendar); 
   if (calendar->conf->UserCS)
   {
// add sub color configs
// Today background
 	face->sub = e_menu_new();
  	sub = e_menu_item_new(face->sub);
 	e_menu_item_label_set(sub, _("Today Background"));
 	TBC_Ptr->calendar = calendar;
   	e_menu_item_callback_set(sub, color_dialog, TBC_Ptr); 
//Today Font Color
   	sub = e_menu_item_new(face->sub);
   	e_menu_item_label_set(sub, _("Today font"));
   	TTC_Ptr->calendar = calendar;
   	e_menu_item_callback_set(sub, color_dialog, TTC_Ptr); 
//weekday background Color
   	sub = e_menu_item_new(face->sub);
   	e_menu_item_label_set(sub, _("WeekDay Background"));
   	WdBC_Ptr->calendar = calendar;
   	e_menu_item_callback_set(sub, color_dialog, WdBC_Ptr); 
//weekday Font Color
   	sub = e_menu_item_new(face->sub);
   	e_menu_item_label_set(sub, _("WeekDay font"));
   	WdTC_Ptr->calendar = calendar;
   	e_menu_item_callback_set(sub, color_dialog, WdTC_Ptr); 
//weekend background Color
   	sub = e_menu_item_new(face->sub);
   	e_menu_item_label_set(sub, _("WeekEnd Background"));
   	WeBC_Ptr->calendar = calendar;
   	e_menu_item_callback_set(sub, color_dialog, WeBC_Ptr); 
//weekend Font Color
   	sub = e_menu_item_new(face->sub);
   	e_menu_item_label_set(sub, _("WeekEnd font"));
   	WeTC_Ptr->calendar = calendar;
   	e_menu_item_callback_set(sub, color_dialog, WeTC_Ptr); 
//Year/Month text Color
   	sub = e_menu_item_new(face->sub);
   	e_menu_item_label_set(sub, _("Year/Month Font Color"));
   	YNC_Ptr->calendar = calendar;
   	e_menu_item_callback_set(sub, color_dialog, YNC_Ptr); 
//weekday Font Color
   	sub = e_menu_item_new(face->sub);
   	e_menu_item_label_set(sub, _("Day Week font Colors"));
   	DTC_Ptr->calendar = calendar;
   	e_menu_item_callback_set(sub, color_dialog, DTC_Ptr); 
// create sub menus for color configs
   	mi = e_menu_item_new(face->menu);
   	e_menu_item_label_set(mi,_("Color Configs"));
   	e_menu_item_submenu_set(mi,face->sub);
//Add option to change fonts
   	face->sub2 = e_menu_new();
   	sub2 = e_menu_item_new(face->sub2);
   	e_menu_item_label_set(sub2, _("Year/Month Text Dialog"));
	YTC_Ptr->calendar = calendar;
   	e_menu_item_callback_set(sub2, font_dialog, YTC_Ptr); 
//Add option to change fonts
   	sub2 = e_menu_item_new(face->sub2);
   	e_menu_item_label_set(sub2, _("Day Text Dialog"));
	DTCs_Ptr->calendar = calendar;
   	e_menu_item_callback_set(sub2, font_dialog, DTCs_Ptr); 
//Add option to change fonts
   	sub2 = e_menu_item_new(face->sub2);
   	e_menu_item_label_set(sub2, _("Text Dialog"));
	TC_Ptr->calendar = calendar;
   	e_menu_item_callback_set(sub2, font_dialog, TC_Ptr); 
// create sub menus for font configs
   	mi = e_menu_item_new(face->menu);
   	e_menu_item_label_set(mi,_("Font Configs"));
   	e_menu_item_submenu_set(mi,face->sub2);
   }
// Add option to select files for background
   face->sub3 = e_menu_new();
   sub3 = e_menu_item_new(face->sub3);
   e_menu_item_label_set(sub3, _("Background EDJ"));
   e_menu_item_callback_set(sub3, file_select_bk_dialog, calendar->conf ); //calendar->conf->Background_path); 
// Add option to select files for background
   sub3 = e_menu_item_new(face->sub3);
   e_menu_item_label_set(sub3, _("TopImage EDJ"));
   e_menu_item_callback_set(sub3, file_select_ti_dialog, calendar->conf); //calendar->conf->TopImage_path); 
   if (calendar->conf->UserCS)
   {
// Add option to select files for background
   	sub3 = e_menu_item_new(face->sub3);
   	e_menu_item_label_set(sub3, _("Config File"));
   	e_menu_item_callback_set(sub3, file_select_cf_dialog, calendar->conf); //calendar->conf->ConfigFile_path); 
   }
// create sub menus for file selections
   mi = e_menu_item_new(face->menu);
   e_menu_item_label_set(mi,_("File Selections"));
   e_menu_item_submenu_set(mi,face->sub3);

}

/***************************************************
/ Function: _calendar_face_cb_gmc_change(void *data, E_Gadman_client *gmc, E_Gadman_Change change)
/ Purpose:  
/ Returns: nothing
/
/*****************************************************/
static void
_calendar_face_cb_gmc_change(void *data, E_Gadman_Client *gmc, 
			    E_Gadman_Change change)
{
   Calendar_Face *face;
   Evas_Coord x, y, w, h;
   
   face = data;
   switch (change)
     {
      case E_GADMAN_CHANGE_MOVE_RESIZE:
	 e_gadman_client_geometry_get(face->gmc, &x, &y, &w, &h);
	 evas_object_move(face->calendar_object, x, y);
	 evas_object_resize(face->calendar_object, w, h);
	 break;
      case E_GADMAN_CHANGE_RAISE:
        evas_object_raise(face->calendar_object);
        evas_object_raise(face->topimage_object);
	break;
      case E_GADMAN_CHANGE_EDGE:
      case E_GADMAN_CHANGE_ZONE:
	 /* FIXME
	  * Must we do something here?
	  */
	 break;
     }
}
/***************************************************
/ Function: 
/ Purpose:  
/
/
/*****************************************************/
static void
_calendar_face_cb_menu_edit(void *data, E_Menu *m, E_Menu_Item *mi)
{
   Calendar_Face *face;

   face = data;
   e_gadman_mode_set(face->gmc->gadman, E_GADMAN_MODE_EDIT);
}
/***************************************************
/ Function: 
/ Purpose:  
/
/
/*****************************************************/
static void 
_calendar_add_dates(Calendar_Face *face, Calendar *calendar, int skew)
{
   time_t now;
   struct tm date;
   int day = 0;
   char buf[3];
   int NumDayMon[12] =  {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
   char TopBuf[256];
   time(&now);
   date = *localtime(&now);
   int DayOfMonth = 1;
   int Sun = 0;
   int Sat = 6;

   snprintf(buf, sizeof(buf), "%d", DayOfMonth);
   switch (calendar->conf->DayofWeek_Start)
   {
	case 0:
		Sat = 6;
		Sun = 0;
		break;
	case 1:
		Sat = 5;
		Sun = 6;
		break;		
	case 2:
		Sat = 4;
		Sun = 5;
		break;
	case 3:
		Sat = 3;
		Sun = 4;
		break;
	case 4:
		Sat = 2;
		Sun = 3;
		break;
	case 5:
		Sat = 1;
		Sun = 2;	
		break;
	case 6:
		Sat = 0;
		Sun = 1;	
		break;
	default:
		break;
   }

   //blank out blocks before day 1
   int i=0;
   for (i;i<skew;i++)
   {
	evas_object_hide(face->weekday_object[i]);
	evas_object_hide(face->weekend_object[i]);
   }
	if (date.tm_mday == 1)
	{
		edje_object_part_text_set(face->today_object[skew], "date-text", buf);
		evas_object_show(face->today_object[skew]);
		evas_object_hide(face->weekend_object[skew]);
		evas_object_hide(face->weekday_object[skew]);
	}
	else
	{
	   switch (skew) //Place the 1st on the proper day
	   {
		case 0: 
			if ((calendar->conf->DayofWeek_Start != 0)||(calendar->conf->DayofWeek_Start != 6))
				edje_object_part_text_set(face->weekday_object[skew], "date-text", buf);
			else
				edje_object_part_text_set(face->weekend_object[skew], "date-text", buf);			
			break;
		case 1:
			if ((calendar->conf->DayofWeek_Start != 5)||(calendar->conf->DayofWeek_Start != 6))
				edje_object_part_text_set(face->weekday_object[skew], "date-text", buf);
			else
				edje_object_part_text_set(face->weekend_object[skew], "date-text", buf);
			break;		
		case 2:
			if ((calendar->conf->DayofWeek_Start != 4)||(calendar->conf->DayofWeek_Start != 5))
				edje_object_part_text_set(face->weekday_object[skew], "date-text", buf);
			else
				edje_object_part_text_set(face->weekend_object[skew], "date-text", buf);	
			break;
		case 3:	
			if ((calendar->conf->DayofWeek_Start != 3)||(calendar->conf->DayofWeek_Start != 4))
				edje_object_part_text_set(face->weekday_object[skew], "date-text", buf);
			else
				edje_object_part_text_set(face->weekend_object[skew], "date-text", buf);		
			break;
		case 4:
			if ((calendar->conf->DayofWeek_Start != 2)||(calendar->conf->DayofWeek_Start != 3))
				edje_object_part_text_set(face->weekday_object[skew], "date-text", buf);
			else
				edje_object_part_text_set(face->weekend_object[skew], "date-text", buf);	
			break;
		case 5:
			if ((calendar->conf->DayofWeek_Start != 1)||(calendar->conf->DayofWeek_Start != 2))
				edje_object_part_text_set(face->weekday_object[skew], "date-text", buf);
			else
				edje_object_part_text_set(face->weekend_object[skew], "date-text", buf);	
			break;
		case 6: 
			if ((calendar->conf->DayofWeek_Start != 0)||(calendar->conf->DayofWeek_Start != 1))
				edje_object_part_text_set(face->weekday_object[skew], "date-text", buf);
			else
				edje_object_part_text_set(face->weekend_object[skew], "date-text", buf);
			break;
		default:
			break;
	   }
	}
	int counter = skew + 1;
	int c_col = counter;
	int row = 0;
	while (DayOfMonth < NumDayMon[date.tm_mon])
	{
	    while ((c_col < 7) && (DayOfMonth <= NumDayMon[date.tm_mon]))
	    {
		DayOfMonth++;
		snprintf(buf, sizeof(buf), "%d", DayOfMonth);
		if (DayOfMonth == date.tm_mday)
		{
			edje_object_part_text_set(face->today_object[counter], "date-text", buf);
			evas_object_show(face->today_object[counter]);
			evas_object_hide(face->weekend_object[counter]);
			evas_object_hide(face->weekday_object[counter]);
		}
		else if ( ((counter % 7)==Sat) || ((counter % 7)==Sun) )
		{
			edje_object_part_text_set(face->weekend_object[counter], "date-text", buf);
		}
		else
			edje_object_part_text_set(face->weekday_object[counter], "date-text", buf);			

		c_col++;
		counter++;
	    } //End inside while
	    c_col = 0;	
	    row++;
	   // 6th week dates..
	   if ((counter > 35) && (DayOfMonth <= NumDayMon[date.tm_mon]))  
	   {
		int counter = counter - 7;
		int week_prior = 0;
		char DateBox[24];
		while (DayOfMonth <= NumDayMon[date.tm_mon])
		{
		   week_prior =  DayOfMonth - 7;
 		   snprintf(DateBox,sizeof(DateBox),"%i/%i",week_prior,DayOfMonth);
		   if (DayOfMonth == date.tm_mday)
		   {
			edje_object_part_text_set(face->today_object[counter], "date-text", DateBox);
			evas_object_show(face->today_object[counter]);
			evas_object_hide(face->weekend_object[counter]);
			evas_object_hide(face->weekday_object[counter]);
	 	   }
		   else if ( ((counter % 7)==Sat) || ((counter % 7)==Sun) )
		   {
			edje_object_part_text_set(face->weekend_object[counter], "date-text", DateBox);
		   }
		   else
		   {
			edje_object_part_text_set(face->weekday_object[counter], "date-text", DateBox);	
		   }
		   DayOfMonth++;
		   counter++;
		}
	   }
 	} //End outside while */
	if (counter < 35)
	{
	   while (counter < 35)
	   {
		evas_object_hide(face->weekday_object[counter]);
		evas_object_hide(face->weekend_object[counter]);
		counter++;
	   }
	}
} //End function


/***************************************************
/ Function: 
/ Purpose:  
/
/
/*****************************************************/
static int
_date_cb_check(void *data)
{
   time_t now;
   Calendar *calendar;
   struct tm date;
   int x;
   int c = 1;
   int row = 1;
   char GridAddress[24];
   char buf[3];

   time(&now);
   date = *localtime(&now);

   calendar = data;
   
   if (calendar->conf->Today == date.tm_mday)
      return 1;
   else
   {
      Evas_List *l;
      for (l = calendar->faces; l; l = l->next)
      {
	Calendar_Face *face;
        face = l->data;
	_fill_in_caltable(calendar->conf->DayofWeek_Start, face, 1);
        _clear_dates(face);
   	if (calendar->conf->UserCS)
   	{
        	update_colors(calendar,face);
	}
	int skew = _calculate_skew(date.tm_mon, calendar);
        _calendar_add_dates(face,calendar,skew);
	set_day_label(face->label_object,calendar->conf->DayofWeek_Start, date.tm_mon, date.tm_year);
      }
   } 
   return 1;
}

/***************************************************
/ Function: 
/ Purpose:  
/
/
/*****************************************************/
static void
_clear_dates(Calendar_Face *face)
{
   int x;

   for (x = 0; x<35;x++)
 	edje_object_part_text_set(face->weekday_object[x], "date-text", " ");
   for (x=0;x<35;x++)
	edje_object_part_text_set(face->weekend_object[x], "date-text", " ");	
   edje_object_part_text_set(face->today_object, "date-text", " ");
}

/***************************************************
/ Function: 
/ Purpose:  
/ Returns: nothing
/ Takes: 
/
/
/
/*****************************************************/
static void 
_read_conf_files(void *data, E_Menu *m, E_Menu_Item *mi)
{
   Calendar *calendar;
   calendar = data;
   calendar_face_set_colors(calendar);
   calendar_face_set_text(calendar);
   redraw_calendar(calendar,0);
}
/***************************************************
/ Function: 
/ Purpose:  
/
/
/*****************************************************/
void 
redraw_calendar(Calendar *calendar, int SwitchImage)
{
  Evas_List *l; 
  Calendar_Face *face;
  int skew;
  time_t now;
  struct tm date;
  time(&now);
  date = *localtime(&now);

  if (SwitchImage == 1)  //Ok, so I need to redraw the whole calendar
  {
     for (l = calendar->faces; l; l = l->next)
     {
		face = l->data;
		_calendar_face_free(face);
		calendar->faces = evas_list_remove(l, face);
      }
      _calendar_face_start(calendar);
  }
  else
  {
     for (l = calendar->faces; l; l = l->next)
     {
     	 face = l->data;
	_fill_in_caltable(calendar->conf->DayofWeek_Start, face, 1);
     	_clear_dates(face);
	skew = _calculate_skew(date.tm_mon,calendar);
     	_calendar_add_dates(face,calendar,skew);  
	set_day_label(face->label_object,calendar->conf->DayofWeek_Start, date.tm_mon, date.tm_year);
	if (calendar->conf->UserCS)
	{
      	   calendar_update_text_fonts(calendar,face);  	 
	   update_colors(calendar,face);
	}
     } 
  }
}
/***************************************************
/ Function: 
/ Purpose:  
/
/
/*****************************************************/
static void
_calendar_face_cb_mouse_down(void *data, Evas *e, Evas_Object *obj,
                            void *event_info)
{
   Calendar_Face *face;
   Evas_Event_Mouse_Down *ev;

   face = data;
   ev = event_info;

   if (ev->button == 3)
     {
        e_menu_activate_mouse(face->menu, e_zone_current_get(face->con),
                              ev->output.x, ev->output.y, 1, 1,
                              E_MENU_POP_DIRECTION_DOWN,ev->timestamp);
        e_util_container_fake_mouse_up_all_later(face->con);
     }
   /*
   else if(ev->button == 1)
     {
 
     }
   */
}

/***************************************************
/ Function: 
/ Purpose:  
/
/
/*****************************************************/
static int _calculate_skew(int month, Calendar *calendar)
{

   time_t now;
   struct tm date;
   time(&now);
   date = *localtime(&now);

   int yearsfrom1900 = date.tm_year;
   int onefourthyear = yearsfrom1900 / 4;
   int onefourthyearmod = yearsfrom1900 % 4;
   int DayofMonth = 1;
   char Year[5];
   int skew; 
   int CurrentYear = date.tm_year + 1900;

   switch (month)
   {
	case 0: //Jan
		skew = (yearsfrom1900 + onefourthyear) % 7;
		if (onefourthyearmod == 0)
			skew = skew -1;
		skew = (skew + 1) % 7;
		break;
	case 1: //Feb
		skew = (yearsfrom1900 + onefourthyear) % 7;
		if (onefourthyearmod == 0)
			skew = skew -1;
		skew = (skew + 1) % 7; //1st day of month
		skew = (skew + 3) % 7; //skew correction for month -  http://klausler.com/new-dayofweek.html
		break;
	case 2: //March
		skew = (yearsfrom1900 + onefourthyear) % 7;
		skew = (skew + 1) % 7; //1st day of month
		skew = (skew + 3) % 7; //skew correction for month -  http://klausler.com/new-dayofweek.html
		break;
	case 3: //April
		skew = (yearsfrom1900 + onefourthyear) % 7;
		skew = (skew + 1) % 7; //1st day of month
		skew = (skew + 3) % 7; //skew correction for month -  http://klausler.com/new-dayofweek.html
		break;
	case 4: //May
		skew = (yearsfrom1900 + onefourthyear) % 7;
		skew = (skew + 1) % 7; //1st day of month
		skew = (skew + 1) % 7; //skew correction for month -  http://klausler.com/new-dayofweek.html
		break;
	case 5: //June
		skew = (yearsfrom1900 + onefourthyear) % 7;
		skew = (skew + 1) % 7; //1st day of month
		skew = (skew + 4) % 7; //skew correction for month -  http://klausler.com/new-dayofweek.html
		break;
	case 6: //July
		skew = (yearsfrom1900 + onefourthyear) % 7;
		skew = (skew + 1) % 7; //1st day of month
		skew = (skew + 6) % 7; //skew correction for month -  http://klausler.com/new-dayofweek.html
		break;
	case 7: //Aug
		skew = (yearsfrom1900 + onefourthyear) % 7;
		skew = (skew + 1) % 7; //1st day of month
		skew = (skew + 2) % 7; //skew correction for month -  http://klausler.com/new-dayofweek.html
		break;
	case 8: //Sept
		skew = (yearsfrom1900 + onefourthyear) % 7;
		skew = (skew + 1) % 7; //1st day of month
		skew = (skew + 5) % 7; //skew correction for month -  http://klausler.com/new-dayofweek.html
		break;
	case 9: //Oct
		skew = (yearsfrom1900 + onefourthyear) % 7;
		skew = (skew + 1) % 7; //1st day of month
		//skew correction for month (none) -  http://klausler.com/new-dayofweek.html
		break;
	case 10: //Nov
		skew = (yearsfrom1900 + onefourthyear) % 7;
		skew = (skew + 1) % 7; //1st day of month
		skew = (skew + 3) % 7; //skew correction for month -  http://klausler.com/new-dayofweek.html
		break;
	case 11: //Dec
		skew = (yearsfrom1900 + onefourthyear) % 7;
		skew = (skew + 1) % 7; //1st day of month
		skew = (skew + 5) % 7; //skew correction for month -  http://klausler.com/new-dayofweek.html

		break;
	default:
		break;
	} //End Switch
	if (calendar->conf->DayofWeek_Start != 0)
	{
	    if (calendar->conf->DayofWeek_Start == 1)
	 	skew = ((skew -1) + 7) % 7;
	    else if (calendar->conf->DayofWeek_Start == 2)
		skew = ((skew - 2) + 7) %7;
	    else if (calendar->conf->DayofWeek_Start == 3)
		skew = ((skew - 3) + 7) %7;
	    else if (calendar->conf->DayofWeek_Start == 4)
		skew = ((skew - 4) + 7) %7;
	    else if (calendar->conf->DayofWeek_Start == 5)
		skew = ((skew - 5) + 7) %7;
	    else if (calendar->conf->DayofWeek_Start == 6)
		skew = ((skew -6) + 7) %7;	
	 }	
	return skew;
}


/***************************************************
/ Function: 
/ Purpose:  
/
/
/*****************************************************/
static void
_fill_in_caltable(int FirstDay, Calendar_Face *face, int StartUp)
{
   int x = 0;
   int i = 0;
   int row = 0;
   while (x<35)
   {
     for (i; i<7;i++)
     {
	   switch (FirstDay)
	   {
		case 0:
			if ((i==0) || (i==6))
			{
				evas_object_show(face->weekend_object[x]);
				evas_object_hide(face->weekday_object[x]);
				evas_object_hide(face->today_object[x]);
			}
			else
			{
				evas_object_show(face->weekday_object[x]);
				evas_object_hide(face->weekend_object[x]);
				evas_object_hide(face->today_object[x]);
			}
			break;
		case 1:
			if ((i==5) || (i==6))
			{
				evas_object_show(face->weekend_object[x]);
				evas_object_hide(face->weekday_object[x]);
				evas_object_hide(face->today_object[x]);
			}
			else
			{
				evas_object_show(face->weekday_object[x]);
				evas_object_hide(face->weekend_object[x]);
				evas_object_hide(face->today_object[x]);
			}
			break;
		case 2:	
			if ((i==4) || (i==5))
			{
				evas_object_show(face->weekend_object[x]);
				evas_object_hide(face->weekday_object[x]);
				evas_object_hide(face->today_object[x]);
			}
			else
			{
				evas_object_show(face->weekday_object[x]);
				evas_object_hide(face->weekend_object[x]);
				evas_object_hide(face->today_object[x]);
			}
			break;
		case 3:	
			if ((i==3) || (i==4))
			{
				evas_object_show(face->weekend_object[x]);
				evas_object_hide(face->weekday_object[x]);
				evas_object_hide(face->today_object[x]);
			}
			else
			{
				evas_object_show(face->weekday_object[x]);
				evas_object_hide(face->weekend_object[x]);
				evas_object_hide(face->today_object[x]);
			}
			break;
		case 4:		
			if ((i==2) || (i==3))
			{
				evas_object_show(face->weekend_object[x]);
				evas_object_hide(face->weekday_object[x]);
				evas_object_hide(face->today_object[x]);
			}
			else
			{
				evas_object_show(face->weekday_object[x]);
				evas_object_hide(face->weekend_object[x]);
				evas_object_hide(face->today_object[x]);
			}
			break;
		case 5:			
			if ((i==1) || (i==2))
			{
				evas_object_show(face->weekend_object[x]);
				evas_object_hide(face->weekday_object[x]);
				evas_object_hide(face->today_object[x]);
			}
			else
			{
				evas_object_show(face->weekday_object[x]);
				evas_object_hide(face->weekend_object[x]);
				evas_object_hide(face->today_object[x]);
			}
			break;
		case 6:
			if ((i==0) || (i==1))
			{
				evas_object_show(face->weekend_object[x]);
				evas_object_hide(face->weekday_object[x]);
				evas_object_hide(face->today_object[x]);
			}
			else
			{
				evas_object_show(face->weekday_object[x]);
				evas_object_hide(face->weekend_object[x]);
				evas_object_hide(face->today_object[x]);
			}
			break;
		default:
			break;
	   }
 	    x++;
     }
     i = 0;
     row++;
   } 

}
